// Brent Cowan Jan. 25 / 2007

#include "GLFW/IncludesGLFW.h"
#include "Extensions.h"
#include "Vector3D.h"
#include <vector>          //standard vectors
#include <fstream>         //Loading and saving files
#include "ModelOBJ.h"

using namespace std;

Vector3D CalculateTan(Vector3D N, Vector3D Va, Vector3D Vb, Vector3D Vc, TextureUV Ta, TextureUV Tb, TextureUV Tc)
{
	static Vector3D side0, side1, tanT, tanS;
	static float deltaS0, deltaS1, deltaT0, deltaT1;

	//Calculate vectors along polygon sides
	side0 = Vc - Va;
	side1 = Vb - Va;

	//Calculate s tangent
	deltaT0 = Tc.v - Ta.v;
	deltaT1 = Tb.v - Ta.v;
	tanS = side0*deltaT1 - side1*deltaT0;
	tanS.Normalize(); tanS.Cross(N); tanS.Normalize();

	//Calculate t tangent
	deltaS0 = Tc.u - Ta.u;
	deltaS1 = Tb.u - Ta.u;
	tanT = side0*deltaS1 - side1*deltaS0;
	tanT.Normalize(); tanT.Cross(N); tanT.Normalize(); tanT.Cross(N);

	side0 = tanT;
	side0.Cross(tanS);
	if(side0.Dot(N)<0.0f)
	{
		tanT = tanT*-1.0f;
	}

	return tanT;
}


ModelOBJ::ModelOBJ()
{
	scale = 1.0f;
	ArrayI = NULL;
	ArrayV = NULL;
	ArrayN = NULL;
	ArrayT = NULL;
	ArrayS = NULL;
}

ModelOBJ::~ModelOBJ()
{
	delete [] ArrayI; ArrayI = NULL;
	delete [] ArrayV; ArrayV = NULL;
	delete [] ArrayN; ArrayN = NULL;
	delete [] ArrayT; ArrayT = NULL;
	delete [] ArrayS; ArrayS = NULL;
}

void ModelOBJ::UnLoad()
{
	scale = 1.0f;
	delete [] ArrayI; ArrayI = NULL;
	delete [] ArrayV; ArrayV = NULL;
	delete [] ArrayN; ArrayN = NULL;
	delete [] ArrayT; ArrayT = NULL;
	delete [] ArrayS; ArrayS = NULL;
}

bool ModelOBJ::AppendFrame(const char * file_name, const char * frame_name)
{
	unsigned int V_size;
	unsigned int F_size;
	unsigned int N_size;
	unsigned int T_size;
	Vector3D*  V; 
	Vector3D*  N;
	Vector3D*  tan;
	TextureUV* T;
	Face3I*    F;
	vector<Vector3D>  vertex;
	vector<Vector3D>  normal;
	vector<TextureUV> texture;
	vector<Face3I>    face;

	//Open the File
	ifstream inFile;
	inFile.open(frame_name, ios::in | ios::_Nocreate);
	if(inFile.fail()){return false;}

	char inChar;
	Vector3D inVector3D;
	TextureUV inUV;
	Face3I inFace;

	//keep loading in arrays until the end of the file is reached
	while( (inChar = inFile.peek()) != EOF )
	{
		inFile >> inChar;
		if(inChar == 'v')
		{
			inChar = inFile.peek();
			if(inChar == ' ')
			{
				inFile >> inVector3D.x >> inVector3D.y >> inVector3D.z;
				inFile.ignore(1);
				inVector3D = inVector3D * scale;
				vertex.push_back(inVector3D);
			}
			else if(inChar == 'n')
			{
				inFile.ignore(1);
				inFile >> inVector3D.x >> inVector3D.y >> inVector3D.z;
				inFile.ignore(1);
				normal.push_back(inVector3D);
			}
			else if(inChar == 't')
			{
				inFile.ignore(1);
				inFile >> inUV.u >> inUV.v;
				inFile.ignore(1);
				texture.push_back(inUV);
			}
		}
		else if(inChar == 'f')
		{
			inChar = inFile.peek();
			if(inChar == ' ')
			{
				inFile >> inFace.va; inFile.ignore(1);
				inFile >> inFace.ta; inFile.ignore(1);
				inFile >> inFace.na; inFile.ignore(1);

				inFile >> inFace.vb; inFile.ignore(1);
				inFile >> inFace.tb; inFile.ignore(1);
				inFile >> inFace.nb; inFile.ignore(1);

				inFile >> inFace.vc; inFile.ignore(1);
				inFile >> inFace.tc; inFile.ignore(1);
				inFile >> inFace.nc; inFile.ignore(1);

				face.push_back(inFace);
			}
		}
		
	}

	inFile.close();

	//Allocate memory to store the model data
	V_size = (unsigned int)vertex.size(); 
	F_size = (unsigned int)face.size();  
	N_size = (unsigned int)normal.size();  
	T_size = (unsigned int)texture.size();  
	V = new Vector3D[V_size];
	N = new Vector3D[N_size];
	tan = new Vector3D[N_size];
	T = new TextureUV[T_size];
	F = new Face3I[F_size];

	//Move model data from vectors into dynamic arrays
	for(unsigned int i = 0; i < V_size; i++) 
	{
		V[i] = vertex[i];
	}

	for(unsigned int i = 0; i < N_size; i++) 
	{
		N[i] = normal[i];
	}

	for(unsigned int i = 0; i < T_size; i++) 
	{
		T[i] = texture[i];
	}

	//Move model data from vectors into dynamic arrays
	for(unsigned int i = 0; i < F_size; i++) 
	{
		F[i] = face[i];
	}

	//Calculate tan
	for(unsigned int i = 0; i < N_size; i++) 
	{
		Vector3D side0, side1, tanT, tanS;
		float deltaS0, deltaS1, deltaT0, deltaT1;

		for(unsigned int k = 0; k < F_size; k++) 
		{
			if(F[k].na-1 == i)
			{
				//Calculate vectors along polygon sides
				side0 =   V[F[k].vc-1]   - V[F[k].va-1];
				side1 =   V[F[k].vb-1]   - V[F[k].va-1];

				//Calculate s tangent
				deltaT0 = T[F[k].tc-1].v - T[F[k].ta-1].v;
				deltaT1 = T[F[k].tb-1].v - T[F[k].ta-1].v;
				tanS = side0*deltaT1 - side1*deltaT0;
				tanS.Normalize(); tanS.Cross(N[F[k].na-1]); tanS.Normalize();

				//Calculate t tangent
				deltaS0 = T[F[k].tc-1].u - T[F[k].ta-1].u;
				deltaS1 = T[F[k].tb-1].u - T[F[k].ta-1].u;
				tanT = side0*deltaS1 - side1*deltaS0;
				tanT.Normalize(); tanT.Cross(N[F[k].na-1]); tanT.Normalize(); tanT.Cross(N[F[k].na-1]);

				//tanS.Cross(tanT); tanS.Normalize();
				//if(tanS.Dot(N[F[k].na-1]) < 0.0f) {tanT.Negate();}
				tan[i] = tanT;// + tanS; tan[i].Normalize();

				k = F_size;
			}
			else if(F[k].nb-1 == i)
			{
				//Calculate vectors along polygon sides
				side0 =   V[F[k].va-1]   - V[F[k].vb-1];
				//side0.Normalize(); //side0.Cross(N[F[k].nb-1]); side0.Normalize(); side0.Cross(N[F[k].nb-1]);
				side1 =   V[F[k].vc-1]   - V[F[k].vb-1];
				//side1.Normalize(); //side1.Cross(N[F[k].nb-1]); side1.Normalize(); side1.Cross(N[F[k].nb-1]);

				//Calculate s tangent
				deltaT0 = T[F[k].ta-1].v - T[F[k].tb-1].v;
				deltaT1 = T[F[k].tc-1].v - T[F[k].tb-1].v;
				tanS = side0*deltaT1 - side1*deltaT0;
				tanS.Normalize();

				//Calculate t tan
				deltaS0 = T[F[k].ta-1].u - T[F[k].tb-1].u;
				deltaS1 = T[F[k].tc-1].u - T[F[k].tb-1].u;
				tanT = side0*deltaS1 - side1*deltaS0;
				tanT.Normalize(); tanT.Cross(N[F[k].nb-1]); tanT.Normalize(); tanT.Cross(N[F[k].nb-1]);

				//tanS.Cross(tanT); tanS.Normalize();
				//if(tanS.Dot(N[F[k].nb-1]) < 0.0f) {tanT.Negate();}
				tan[i] = tanT;// + tanS; tan[i].Normalize();

				k = F_size;
			}
			else if(F[k].nc-1 == i)
			{
				//Calculate vectors along polygon sides
				side0 =   V[F[k].vb-1]   - V[F[k].vc-1];
				side1 =   V[F[k].va-1]   - V[F[k].vc-1];

				//Calculate s tangent
				deltaT0 = T[F[k].tb-1].v - T[F[k].tc-1].v;
				deltaT1 = T[F[k].ta-1].v - T[F[k].tc-1].v;
				tanS = side0*deltaT1 - side1*deltaT0;
				tanS.Normalize(); //tanS.Cross(N[F[k].nc-1]); tanS.Normalize();

				//Calculate T tangent
				deltaS0 = T[F[k].tb-1].u - T[F[k].tc-1].u;
				deltaS1 = T[F[k].ta-1].u - T[F[k].tc-1].u;
				tanT = side0*deltaS1 - side1*deltaS0;
				tanT.Normalize(); tanT.Cross(N[F[k].nc-1]); tanT.Normalize(); tanT.Cross(N[F[k].nc-1]);

				//tanS.Cross(tanT); tanS.Normalize();
				//if(tanS.Dot(N[F[k].nc-1]) < 0.0f) {tanT.Negate();}
				tan[i] = tanT;// + tanS; tan[i].Normalize();

				k = F_size;
			}
		}
	}

	//dealocate memory
	delete [] ArrayV; ArrayV = NULL;
	delete [] ArrayN; ArrayN = NULL;
	delete [] ArrayT; ArrayT = NULL;
	delete [] ArrayS; ArrayS = NULL;

	//Allocate memory
	//ArrayI = new GLushort[sizeI]; //Stays the same
	ArrayV = new GLfloat[sizeV*3];
	ArrayN = new GLfloat[sizeV*3];
	ArrayT = new GLfloat[sizeV*2];
	ArrayS = new GLfloat[sizeV*3];

	//Transfer data to arrays
	//for(GLushort i = 0; i < sizeI; i++)
	//{
	//	ArrayI[i] = index[i];
	//}

	boxMin = vertex[0];
	boxMax = vertex[0];
	for(GLushort i = 0; i < sizeV; i++)
	{
		ArrayV[i*3] = V[vertexO[i]].x; ArrayV[i*3+1] = V[vertexO[i]].y; ArrayV[i*3+2] = V[vertexO[i]].z;
		ArrayN[i*3] = N[normalO[i]].x; ArrayN[i*3+1] = N[normalO[i]].y; ArrayN[i*3+2] = N[normalO[i]].z;
		ArrayS[i*3] = tan[normalO[i]].x; ArrayS[i*3+1] = tan[normalO[i]].y; ArrayS[i*3+2] = tan[normalO[i]].z;
		ArrayT[i*2] = T[textureO[i]].u; ArrayT[i*2+1] = T[textureO[i]].v;

		//Update box size
		if(boxMin.x > V[vertexO[i]].x)boxMin.x=V[vertexO[i]].x;
		if(boxMin.y > V[vertexO[i]].y)boxMin.y=V[vertexO[i]].y;
		if(boxMin.z > V[vertexO[i]].z)boxMin.z=V[vertexO[i]].z;

		if(boxMax.x < V[vertexO[i]].x)boxMax.x=V[vertexO[i]].x;
		if(boxMax.y < V[vertexO[i]].y)boxMax.y=V[vertexO[i]].y;
		if(boxMax.z < V[vertexO[i]].z)boxMax.z=V[vertexO[i]].z;
	}

	delete [] V; V = NULL;
	delete [] N; N = NULL;
	delete [] tan; tan = NULL;
	delete [] T; T = NULL;
	delete [] F; F = NULL;

	//return true; //Got this far, must have worked! // Not yet!
	//First we have to append this frame to the G3D file!

	//Open the File
	ofstream out;
	unsigned char temp = 0;
	unsigned short tempShort = 0;
	Vector3D tempVec;

	short int minX, minY, minZ, maxX, maxY, maxZ;

	minX = short(boxMin.x);
	minY = short(boxMin.y);
	minZ = short(boxMin.z);
	maxX = short(boxMax.x)+1;
	maxY = short(boxMax.y)+1;
	maxZ = short(boxMax.z)+1;

	//Create the file
	out.open(file_name, ios::out | ios::binary | ios::_Nocreate);
	if(out.fail()){return false;}

	out.seekp(0, ios::end); // End of file

	//Output the Box size
	// Box min  2 + 2 + 2
	// Box max  2 + 2 + 2

	tempShort = minX + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = minY + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = minZ + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = maxX + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = maxY + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = maxZ + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);

	//Output the Vertex data
	// Vertex  6
	// Normal  2
	// Tangent 2
	for(GLushort i = 0; i < sizeV; i++)
	{
		//Output Vertex X
		tempShort = GLushort(((ArrayV[i*3] - float(minX)) / (float(maxX-minX))) * 65535.0f + 0.5f);
		temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
		//Output Vertex Y
		tempShort = GLushort(((ArrayV[i*3+1] - float(minY)) / (float(maxY-minY))) * 65535.0f + 0.5f);
		temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
		//Output Vertex Z
		tempShort = GLushort(((ArrayV[i*3+2] - float(minZ)) / (float(maxZ-minZ))) * 65535.0f + 0.5f);
		temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);

		//Output Normal
		tempVec.x = ArrayN[i*3]; tempVec.y = ArrayN[i*3+1]; tempVec.z = ArrayN[i*3+2];
		tempVec = tempVec.vectorToPolar(tempVec);
		tempShort = unsigned short(tempVec.x/360.0f*256.0f); 
		temp = unsigned char(tempShort%256);
		out.write((char*)&temp, 1); 
		tempShort = unsigned short(tempVec.y/360.0f*256.0f); 
		temp = unsigned char(tempShort%256);
		out.write((char*)&temp, 1);

		//Output Tangent
		tempVec.x = ArrayS[i*3]; tempVec.y = ArrayS[i*3+1]; tempVec.z = ArrayS[i*3+2];
		tempVec = tempVec.vectorToPolar(tempVec);
		temp = unsigned char(tempVec.x/360.0f*256.0f); 
		out.write((char*)&temp, 1); 
		temp = unsigned char(tempVec.y/360.0f*256.0f); 
		out.write((char*)&temp, 1);
	}

	out.close(); //Close file

	//Got this far, must have worked
	return true;
}

bool ModelOBJ::Load(const char * file_name)
{
	unsigned int V_size;
	unsigned int F_size;
	unsigned int N_size;
	unsigned int T_size;
	Vector3D*  V; 
	Vector3D*  N;
	Vector3D*  tan;
	TextureUV* T;
	Face3I*    F;
	vector<Vector3D>  vertex;
	vector<Vector3D>  normal;
	vector<TextureUV> texture;
	vector<Face3I>    face;

	//Open the File
	ifstream inFile;
	inFile.open(file_name, ios::in | ios::_Nocreate);
	if(inFile.fail()){return false;}

	char inChar;
	Vector3D inVector3D;
	TextureUV inUV;
	Face3I inFace;

	//keep loading in arrays until the end of the file is reached
	while( (inChar = inFile.peek()) != EOF )
	{
		inFile >> inChar;
		if(inChar == 'v')
		{
			inChar = inFile.peek();
			if(inChar == ' ')
			{
				inFile >> inVector3D.x >> inVector3D.y >> inVector3D.z;
				inFile.ignore(1);
				inVector3D = inVector3D * scale;
				vertex.push_back(inVector3D);
			}
			else if(inChar == 'n')
			{
				inFile.ignore(1);
				inFile >> inVector3D.x >> inVector3D.y >> inVector3D.z;
				inFile.ignore(1);
				normal.push_back(inVector3D);
			}
			else if(inChar == 't')
			{
				inFile.ignore(1);
				inFile >> inUV.u >> inUV.v;
				inFile.ignore(1);
				texture.push_back(inUV);
			}
		}
		else if(inChar == 'f')
		{
			inChar = inFile.peek();
			if(inChar == ' ')
			{
				inFile >> inFace.va; inFile.ignore(1);
				inFile >> inFace.ta; inFile.ignore(1);
				inFile >> inFace.na; inFile.ignore(1);

				inFile >> inFace.vb; inFile.ignore(1);
				inFile >> inFace.tb; inFile.ignore(1);
				inFile >> inFace.nb; inFile.ignore(1);

				inFile >> inFace.vc; inFile.ignore(1);
				inFile >> inFace.tc; inFile.ignore(1);
				inFile >> inFace.nc; inFile.ignore(1);

				face.push_back(inFace);
			}
		}
		
	}

	inFile.close();

	//Allocate memory to store the model data
	V_size = (unsigned int)vertex.size();  
	F_size = (unsigned int)face.size();  
	N_size = (unsigned int)normal.size();  
	T_size = (unsigned int)texture.size();  
	V = new Vector3D[V_size];
	N = new Vector3D[N_size];
	tan = new Vector3D[N_size];
	T = new TextureUV[T_size];
	F = new Face3I[F_size];

	//Move model data from vectors into dynamic arrays
	for(unsigned int i = 0; i < V_size; i++) 
	{
		V[i] = vertex[i];
	}

	for(unsigned int i = 0; i < N_size; i++) 
	{
		N[i] = normal[i];
	}

	for(unsigned int i = 0; i < T_size; i++) 
	{
		T[i] = texture[i];
	}

	//Move model data from vectors into dynamic arrays
	for(unsigned int i = 0; i < F_size; i++) 
	{
		F[i] = face[i];
	}

	//Convert all data to vertex arrays
	vertex.clear();
	normal.clear();
	texture.clear();
	face.clear();
	vector<GLushort> index;
	vector<Vector3D> tangent;
	GLushort tempIndex;

	for(unsigned int i = 0; i < F_size; i++)
	{
		GLushort k;
		for(k = 0; k < vertex.size(); k++)
		{
			//if this vertex/norm/tex equals an exsisting entry
			if(V[F[i].va-1] == vertex[k] && N[F[i].na-1] == normal[k] && T[F[i].ta-1].u == texture[k].u && T[F[i].ta-1].v == texture[k].v)
			{
				tempIndex = k;
				index.push_back(tempIndex);
				k = 65000; //we found a match, we're done
			}
		}
		//if we didn't find a match, make a new entry
		if(k != 65001)
		{
			tempIndex = GLushort(vertex.size());
			index.push_back(tempIndex);
			vertex.push_back(V[F[i].va-1]);
			normal.push_back(N[F[i].na-1]);
			texture.push_back(T[F[i].ta-1]);
			tangent.push_back(tan[F[i].na-1]);
			//////////////////////////
			vertexO.push_back(F[i].va-1);
			normalO.push_back(F[i].na-1);
			textureO.push_back(F[i].ta-1);
		}

		for(k = 0; k < vertex.size(); k++)
		{
			//if this vertex/norm/tex equals an exsisting entry
			if(V[F[i].vb-1] == vertex[k] && N[F[i].nb-1] == normal[k] && T[F[i].tb-1].u == texture[k].u && T[F[i].tb-1].v == texture[k].v)
			{
				tempIndex = k;
				index.push_back(tempIndex);
				k = 65000; //we found a match, we're done
			}
		}
		//if we didn't find a match, make a new entry
		if(k != 65001)
		{
			tempIndex = GLushort(vertex.size());
			index.push_back(tempIndex);
			vertex.push_back(V[F[i].vb-1]);
			normal.push_back(N[F[i].nb-1]);
			texture.push_back(T[F[i].tb-1]);
			tangent.push_back(tan[F[i].nb-1]);
			//////////////////////////
			vertexO.push_back(F[i].vb-1);
			normalO.push_back(F[i].nb-1);
			textureO.push_back(F[i].tb-1);
		}

		for(k = 0; k < vertex.size(); k++)
		{
			//if this vertex/norm/tex equals an exsisting entry
			if(V[F[i].vc-1] == vertex[k] && N[F[i].nc-1] == normal[k] && T[F[i].tc-1].u == texture[k].u && T[F[i].tc-1].v == texture[k].v)
			{
				tempIndex = k;
				index.push_back(tempIndex);
				k = 65000; //we found a match, we're done
			}
		}
		//if we didn't find a match, make a new entry
		if(k != 65001)
		{
			tempIndex = GLushort(vertex.size());
			index.push_back(tempIndex);
			vertex.push_back(V[F[i].vc-1]);
			normal.push_back(N[F[i].nc-1]);
			texture.push_back(T[F[i].tc-1]);
			tangent.push_back(tan[F[i].nc-1]);
			//////////////////////////
			vertexO.push_back(F[i].vc-1);
			normalO.push_back(F[i].nc-1);
			textureO.push_back(F[i].tc-1);
		}
	}

	//Copy from the vectors to the arrays
	sizeI = (GLuint)index.size();
	sizeV = (GLuint)vertex.size();

	//Allocate memory
	ArrayI = new GLuint[sizeI];
	ArrayV = new GLfloat[sizeV*3];
	ArrayN = new GLfloat[sizeV*3];
	ArrayT = new GLfloat[sizeV*2];
	ArrayS = new GLfloat[sizeV*3];

	//Transfer data to arrays
	for(GLuint i = 0; i < sizeI; i+=3)
	{
		ArrayI[i]   = index[i];
		ArrayI[i+1] = index[i+1];
		ArrayI[i+2] = index[i+2];

		//Calculate Tangent
		tangent[index[i]]   = CalculateTan(normal[index[i]],   vertex[index[i]],   vertex[index[i+1]], vertex[index[i+2]], texture[index[i]],   texture[index[i+1]], texture[index[i+2]]);
		tangent[index[i+1]] = CalculateTan(normal[index[i+1]], vertex[index[i+1]], vertex[index[i+2]], vertex[index[i]],   texture[index[i+1]], texture[index[i+2]], texture[index[i]]);
		tangent[index[i+2]] = CalculateTan(normal[index[i+2]], vertex[index[i+2]], vertex[index[i]],   vertex[index[i+1]], texture[index[i+2]], texture[index[i]],   texture[index[i+1]]);
	}

	boxMin = vertex[0];
	boxMax = vertex[0];
	for(GLuint i = 0; i < sizeV; i++)
	{
		ArrayV[i*3] = vertex[i].x; ArrayV[i*3+1] = vertex[i].y; ArrayV[i*3+2] = vertex[i].z;
		ArrayN[i*3] = normal[i].x; ArrayN[i*3+1] = normal[i].y; ArrayN[i*3+2] = normal[i].z;
		ArrayS[i*3] = tangent[i].x; ArrayS[i*3+1] = tangent[i].y; ArrayS[i*3+2] = tangent[i].z;
		ArrayT[i*2] = texture[i].u; ArrayT[i*2+1] = texture[i].v;

		//Update box size
		if(boxMin.x > vertex[i].x)boxMin.x=vertex[i].x;
		if(boxMin.y > vertex[i].y)boxMin.y=vertex[i].y;
		if(boxMin.z > vertex[i].z)boxMin.z=vertex[i].z;

		if(boxMax.x < vertex[i].x)boxMax.x=vertex[i].x;
		if(boxMax.y < vertex[i].y)boxMax.y=vertex[i].y;
		if(boxMax.z < vertex[i].z)boxMax.z=vertex[i].z;
	}

	delete [] V; V = NULL;
	delete [] N; N = NULL;
	delete [] tan; tan = NULL;
	delete [] T; T = NULL;
	delete [] F; F = NULL;

	return true; //Got this far, must have worked!
}

void ModelOBJ::Render()
{
	if(ArrayI!=NULL)
	{
		glVertexPointer(3, GL_FLOAT, 0, &ArrayV[0]);
		glNormalPointer(GL_FLOAT, 0, &ArrayN[0]);
		glTexCoordPointer(2, GL_FLOAT, 0, &ArrayT[0]);
		ext::glSecondaryColorPointerEXT(3, GL_FLOAT, 0, &ArrayS[0]);
		glDrawElements(GL_TRIANGLES, sizeI, GL_UNSIGNED_INT, &ArrayI[0]);
		
		//Draw min and max
		/*glPointSize(10.0f); 
		glBegin(GL_POINTS); 
			glVertex3f(boxMin.x, boxMin.y, boxMin.z); 
			glVertex3f(boxMax.x, boxMax.y, boxMax.z); 
		glEnd();*/

		/*glBegin(GL_TRIANGLES); 
		for(int i = 0; i < sizeI; i++)
		{
			ext::glMultiTexCoord2fARB(GL_TEXTURE0_ARB, ArrayT[ArrayI[i]*2], ArrayT[ArrayI[i]*2+1]);
			glNormal3f(           ArrayN[ArrayI[i]*3], ArrayN[ArrayI[i]*3+1], ArrayN[ArrayI[i]*3+2]);
			ext::glSecondaryColor3fEXT(ArrayS[ArrayI[i]*3], ArrayS[ArrayI[i]*3+1], ArrayS[ArrayI[i]*3+2]);
			glVertex3f(           ArrayV[ArrayI[i]*3], ArrayV[ArrayI[i]*3+1], ArrayV[ArrayI[i]*3+2]); 
		}
		glEnd();*/
	}
}

bool ModelOBJ::ExportG3D(const char * file_name)
{
	//Open the File
	ofstream out;
	unsigned char temp = 0;
	unsigned short tempShort = 0;
	Vector3D tempVec;

	short int minX, minY, minZ, maxX, maxY, maxZ;

	minX = short(boxMin.x);
	minY = short(boxMin.y);
	minZ = short(boxMin.z);
	maxX = short(boxMax.x)+1;
	maxY = short(boxMax.y)+1;
	maxZ = short(boxMax.z)+1;

	//Create the file
	out.open(file_name, ios::out | ios::binary);
	if(out.fail()){return false;}

	//Output the header
	// Index size  2
	// Vertexes    2
	// Frames      2

	temp = unsigned char((sizeI) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((sizeI) % 256); out.write((char*)&temp, 1);

	temp = unsigned char((sizeV) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((sizeV) % 256); out.write((char*)&temp, 1);
	
	temp = unsigned char((frames) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((frames) % 256); out.write((char*)&temp, 1);

	//Output the Index
	// Index element  2

	for(GLuint i = 0; i < sizeI; i++)
	{
		temp = unsigned char((ArrayI[i]) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((ArrayI[i]) % 256); out.write((char*)&temp, 1);
	}

	//Output Texture Coordinates
	// Texture 3
	for(GLuint i = 0; i < sizeV; i++)
	{
		unsigned char tempTex;
		tempShort = unsigned short(ArrayT[i*2]*4095.0f+0.5f);
		temp = unsigned char((tempShort) % 256);
		tempTex = unsigned char(tempShort/256) *16;
		out.write((char*)&temp, 1); 

		tempShort = unsigned short(ArrayT[i*2+1]*4095.0f+0.5f); 
		temp = unsigned char((tempShort) % 256);
		tempTex += unsigned char(tempShort/256);
		out.write((char*)&temp, 1); 
		out.write((char*)&tempTex, 1);
	}

	//Output the Box size
	// Box min  2 + 2 + 2
	// Box max  2 + 2 + 2

	tempShort = minX + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = minY + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = minZ + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = maxX + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = maxY + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
	tempShort = maxZ + 32768;
	temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
	temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);

	//Output the Vertex data
	// Vertex  6
	// Normal  2
	// Tangent 2
	for(GLushort i = 0; i < sizeV; i++)
	{
		//Output Vertex X
		tempShort = GLushort(((ArrayV[i*3] - float(minX)) / (float(maxX-minX))) * 65535.0f + 0.5f);
		temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
		//Output Vertex Y
		tempShort = GLushort(((ArrayV[i*3+1] - float(minY)) / (float(maxY-minY))) * 65535.0f + 0.5f);
		temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);
		//Output Vertex Z
		tempShort = GLushort(((ArrayV[i*3+2] - float(minZ)) / (float(maxZ-minZ))) * 65535.0f + 0.5f);
		temp = unsigned char((tempShort) / 256); out.write((char*)&temp, 1);
		temp = unsigned char((tempShort) % 256); out.write((char*)&temp, 1);

		//Output Normal
		tempVec.x = ArrayN[i*3]; tempVec.y = ArrayN[i*3+1]; tempVec.z = ArrayN[i*3+2];
		tempVec = tempVec.vectorToPolar(tempVec);
		tempShort = unsigned short(tempVec.x/360.0f*256.0f); 
		temp = unsigned char(tempShort%256);
		out.write((char*)&temp, 1); 
		tempShort = unsigned short(tempVec.y/360.0f*256.0f); 
		temp = unsigned char(tempShort%256);
		out.write((char*)&temp, 1);

		//Output Tangent
		tempVec.x = ArrayS[i*3]; tempVec.y = ArrayS[i*3+1]; tempVec.z = ArrayS[i*3+2];
		tempVec = tempVec.vectorToPolar(tempVec);
		temp = unsigned char(tempVec.x/360.0f*256.0f); 
		out.write((char*)&temp, 1); 
		temp = unsigned char(tempVec.y/360.0f*256.0f); 
		out.write((char*)&temp, 1);
	}

	out.close(); //Close file

	//Got this far, must have worked
	return true;
}